
local format = string.format
local floor = math.floor
local min = math.min

local GetFormattedTime = function(s)
	if s >= 86400 then
		return format('%dd', floor(s/86400 + 0.5)), s % 86400
	elseif s >= 3600 then
		return format('%dh', floor(s/3600 + 0.5)), s % 3600
	elseif s >= 60 then
		return format('%dm', floor(s/60 + 0.5)), s % 60
	end
	return floor(s + 0.5), s - floor(s)
end

local UpdateTimer = function(self, elapsed)
	if self.text:IsShown() then
		if self.nextUpdate > 0 then
			self.nextUpdate = self.nextUpdate - elapsed
		else
			local remain = self.duration - (GetTime() - self.start)
			if floor(remain + 0.5) > 0 then
				local time, nextUpdate = GetFormattedTime(remain)
				self.text:SetText(time)
				self.nextUpdate = nextUpdate
			else
				self.text:Hide()
			end
		end
	end
end

local HideTimer = function(self)
	if self.text then
		self.text:Hide()
	end
end

local CreateTimer = function(self)
	local text = self:CreateFontString(nil, 'OVERLAY')
	text:SetPoint('CENTER', 1, 0)
	text:SetFont("Interface\\Addons\\alCooltimer\\Hooge 06_65.ttf", 8, 'OUTLINEMONOCHROME')--GameFontNormal:GetFont(), floor(14 * self:GetParent():GetWidth() / 26), 'OUTLINE')
	text:SetTextColor(1, 1, 1)
	self.text = text
	return text
end

local StartTimer = function(self, start, duration)
	local text = self.text or CreateTimer(self)
	if text then
		self.start = start
		self.duration = duration
		self.nextUpdate = 0
		self:SetScript('OnUpdate', UpdateTimer)
		text:Show()
	end
end

local methods = getmetatable(CreateFrame("Cooldown")).__index
hooksecurefunc(methods, 'SetCooldown', function(self, start, duration)
	if start > 0 and duration > 3 and not self.noCooldownCount then
		StartTimer(self, start, duration)
	else
		HideTimer(self)
	end
end)
